
/***************************************************************************
 *   This file is part of Aspect, a simple PEC tool.                       *
 *                                                                         *
 *   Copyright (C) 2007 by Wolfgang Hoffmann <woho@woho.de>                *
 *                                                                         *
 *   This program is free software, licensed under the GPL v2.             *
 *   See the file COPYING for more details.                                *
 ***************************************************************************/


#ifndef ASPECT_DATAPEMCU_H
#define ASPECT_DATAPEMCU_H


#include <QFile>
#include <QList>
#include <QObject>
#include <QPointer>
#include <QStandardItemModel>
#include <QTime>
#include <QTimer>
#include <QVector>


class PecData;


class PeRawSample
    {
public:
    QTime m_oTime;
    double m_dfWorm;
    double m_dfRa;
    double m_dfDec;
    };


class PePureSample
    {
public:
    double m_dfWorm;
    double m_dfPe;
    };


class Period
    {
public:
    int m_nRowBeg;
    int m_nRowEnd;
    double m_dfPeMin;
    double m_dfPeMax;
    };


class DataPeMcu : public QObject
    {
    Q_OBJECT

public:
    DataPeMcu();
    ~DataPeMcu();

    void clearData();
    bool probePeFile(const QString &qsFileName) const;
    void setPeFileName(const QString &qsFileName);
    bool loadPeFile();
    bool pollPeFile(bool bOn);

    void processData(int nBeg, int nEnd, bool bAverage, int nSmoothWidth,
        double dfDrift, double dfScale, double dfOffset,
        PecData *pPecTable, int nWormWheelTeeth, double dfPecPosScale);

    QStandardItemModel *model() { return m_qpModel; };
    QList<PeRawSample> samples() const { return m_lstPeRaw; };
    QList<Period> periods() const { return m_lstPeriod; };
    QList<PePureSample> peSmoothed() const { return m_lstPeSmoothed; };
    QVector<double> peProcessed() const { return m_aPeProcessed; };
    QVector<double> pecIdeal() const { return m_aPecIdeal; };

signals:
    void dataPeChanged();

private:
    bool openPeFile();
    void closePeFile();
    void clearPeData();

private slots:
    void readPeFile();
    void changePeItem(QStandardItem *pItem);

private:
    QFile m_oPeFile;
    QTimer m_oPoller;
    QString m_qsLineFragment;
    bool m_bInModelRefill;
    QPointer<QStandardItemModel> m_qpModel;
    QList<PeRawSample> m_lstPeRaw;
    QList<PePureSample> m_lstPePure;
    QList<PePureSample> m_lstPeSmoothed;
    QList<Period> m_lstPeriod;
    QVector<double> m_aPeProcessed;
    QVector<double> m_aPecIdeal;
    };


#endif // ASPECT_DATAPEMCU_H

