
/***************************************************************************
 *   This file is part of Aspect, a simple PEC tool.                       *
 *                                                                         *
 *   Copyright (C) 2007 by Wolfgang Hoffmann <woho@woho.de>                *
 *                                                                         *
 *   This program is free software, licensed under the GPL v2.             *
 *   See the file COPYING for more details.                                *
 ***************************************************************************/


#include "logwindow.h"
#include "ui_logwindow.h"
#include "log.h"

#include <QtGui>




/********************************************************************/
/*!
    \class LogDialog
    \brief Dialog for viewing log of communication with MCU

*/

LogDialog::LogDialog(QWidget *pParent)
:   QDialog(pParent)
    {
    setAttribute(Qt::WA_DeleteOnClose, false);
    setAttribute(Qt::WA_QuitOnClose, false);

    m_pUi = new Ui::LogDialog;
    m_pUi->setupUi(this);

    // add empty last line for incomplete lines and autoscrolling
    m_pUi->logView->addItem(QString());
    m_pUi->logView->setCurrentRow(0);

    Log::logConnect(this, SLOT(append(QString, bool)));
    //Log::mcuConnect(this, SLOT(append(QString, bool)));
    resize(800, 600);
    }


LogDialog::~LogDialog()
    {
    delete m_pUi;
    }


void LogDialog::append(const QString &qsText, bool bComplete)
    {
    int nCount = m_pUi->logView->count();
    QListWidgetItem *pItem = m_pUi->logView->item(nCount - 1);
    if (pItem == 0)
        return;
    QString qsItemText = pItem->text() + qsText;
    if (bComplete)
        {
        m_pUi->logView->insertItem(nCount - 1, qsItemText);
        // limit to 200 lines, otherwise performance suffers
        if (nCount > 200)
            delete m_pUi->logView->takeItem(0);
        m_pUi->logView->scrollToItem(m_pUi->logView->currentItem());
        qsItemText = QString();
        }
    pItem->setText(qsItemText);
    }

